@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -33,13 +33,13 @@
 /// @file PointScatter.h
 ///
 /// @brief We offer three different algorithms (each in its own class)
-///        for scattering of point in active voxels:
+///        for scattering of points in active voxels:
 ///
 /// 1) UniformPointScatter. Has two modes: Either randomly distributes
-///    a fixed number of points in the active voxels, or the user can
+///    a fixed number of points into the active voxels, or the user can
 ///    specify a fixed probability of having a points per unit of volume.
 ///
-/// 2) DenseUniformPointScatter. Randomly distributes points in active
+/// 2) DenseUniformPointScatter. Randomly distributes points into active
 ///    voxels using a fixed number of points per voxel.
 ///
 /// 3) NonIniformPointScatter. Define the local probability of having
@@ -99,7 +99,7 @@ class BasePointScatter;
 /// interrupter calls are no-ops (i.e. incurs no computational overhead).
 
 
-/// @brief Uniform scatters of point in the active voxels.
+/// @brief Uniformly scatters points in the active voxels.
 /// The point count is either explicitly defined or implicitly
 /// through the specification of a global density (=points-per-volume)
 ///
@@ -119,8 +119,9 @@ public:
     UniformPointScatter(PointAccessorType& points,
                         Index64 pointCount,
                         RandomGenerator& randGen,
+                        double spread = 1.0,
                         InterruptType* interrupt = NULL)
-        : BaseT(points, randGen, interrupt)
+        : BaseT(points, randGen, spread, interrupt)
         , mTargetPointCount(pointCount)
         , mPointsPerVolume(0.0f)
     {
@@ -128,8 +129,9 @@ public:
     UniformPointScatter(PointAccessorType& points,
                         float pointsPerVolume,
                         RandomGenerator& randGen,
+                        double spread = 1.0,
                         InterruptType* interrupt = NULL)
-        : BaseT(points, randGen, interrupt)
+        : BaseT(points, randGen, spread, interrupt)
         , mTargetPointCount(0)
         , mPointsPerVolume(pointsPerVolume)
     {
@@ -157,10 +159,11 @@ public:
         math::RandInt<Index64, RandomGenerator> rand(BaseT::mRand01.engine(), 0, mVoxelCount-1);
         for (Index64 i=0; i<mTargetPointCount; ++i) list[i] = rand();
         tbb::parallel_sort(list.get(), list.get() + mTargetPointCount);
-
+        
         CoordBBox bbox;
         const Vec3R offset(0.5, 0.5, 0.5);
         typename GridT::ValueOnCIter valueIter = grid.cbeginValueOn();
+        
         for (Index64 i=0, n=valueIter.getVoxelCount() ; i != mTargetPointCount; ++i) {
             if (BaseT::interrupt()) return false;
             const Index64 voxelId = list[i];
@@ -172,9 +175,10 @@ public:
                 BaseT::addPoint(grid, valueIter.getCoord() - offset);
             } else {// tiles contain multiple (virtual) voxels
                 valueIter.getBoundingBox(bbox);
-                BaseT::addPoint(grid, bbox.min() - offset, bbox.extents());
+                BaseT::addPoint(grid, bbox.min() - offset, bbox.extents()); 
             }
-        }//loop over all the active voxels and tiles
+        }//loop over all the active voxels and tiles 
+        //}
 
         BaseT::end();
         return true;
@@ -216,8 +220,9 @@ public:
     DenseUniformPointScatter(PointAccessorType& points,
                              float pointsPerVoxel,
                              RandomGenerator& randGen,
+                             double spread = 1.0,
                              InterruptType* interrupt = NULL)
-        : BaseT(points, randGen, interrupt)
+        : BaseT(points, randGen, spread, interrupt)
         , mPointsPerVoxel(pointsPerVoxel)
     {
     }
@@ -237,13 +242,13 @@ public:
         const int ppv = math::Floor(mPointsPerVoxel);
         const double delta = mPointsPerVoxel - ppv;
         const bool fractional = !math::isApproxZero(delta, 1.0e-6);
-
+        
         for (ValueIter iter = grid.cbeginValueOn(); iter; ++iter) {
             if (BaseT::interrupt()) return false;
             if (iter.isVoxelValue()) {// a majority is expected to be voxels
                 const Vec3R dmin = iter.getCoord() - offset;
                 for (int n = 0; n != ppv; ++n) BaseT::addPoint(grid, dmin);
-                if (fractional && BaseT::getRand() < delta) BaseT::addPoint(grid, dmin);
+                if (fractional && BaseT::getRand01() < delta) BaseT::addPoint(grid, dmin);
             } else {// tiles contain multiple (virtual) voxels
                 iter.getBoundingBox(bbox);
                 const Coord size(bbox.extents());
@@ -251,10 +256,10 @@ public:
                 const double d = mPointsPerVoxel * iter.getVoxelCount();
                 const int m = math::Floor(d);
                 for (int n = 0; n != m; ++n)  BaseT::addPoint(grid, dmin, size);
-                if (BaseT::getRand() < d - m) BaseT::addPoint(grid, dmin, size);
+                if (BaseT::getRand01() < d - m) BaseT::addPoint(grid, dmin, size);
             }
         }//loop over all the active voxels and tiles
-
+        //}
         BaseT::end();
         return true;
     }
@@ -297,8 +302,9 @@ public:
     NonUniformPointScatter(PointAccessorType& points,
                            float pointsPerVolume,
                            RandomGenerator& randGen,
+                           double spread = 1.0,
                            InterruptType* interrupt = NULL)
-        : BaseT(points, randGen, interrupt)
+        : BaseT(points, randGen, spread, interrupt)
         , mPointsPerVolume(pointsPerVolume)//note this is merely a
                                            //multiplier for the local point density
     {
@@ -324,13 +330,13 @@ public:
             if (iter.isVoxelValue()) { // a majority is expected to be voxels
                 const Vec3R dmin =iter.getCoord() - offset;
                 for (int i = 0; i < n; ++i) BaseT::addPoint(grid, dmin);
-                if (BaseT::getRand() < (d - n)) BaseT::addPoint(grid, dmin);
+                if (BaseT::getRand01() < (d - n)) BaseT::addPoint(grid, dmin);
             } else { // tiles contain multiple (virtual) voxels
                 iter.getBoundingBox(bbox);
                 const Coord size(bbox.extents());
                 const Vec3R dmin = bbox.min() - offset;
                 for (int i = 0; i < n; ++i) BaseT::addPoint(grid, dmin, size);
-                if (BaseT::getRand() < (d - n)) BaseT::addPoint(grid, dmin, size);
+                if (BaseT::getRand01() < (d - n)) BaseT::addPoint(grid, dmin, size);
             }
         }//loop over all the active voxels and tiles
         BaseT::end();
@@ -367,26 +373,29 @@ public:
 
 protected:
 
+    PointAccessorType&        mPoints;
+    InterruptType*            mInterrupter;
+    Index64                   mPointCount;
+    Index64                   mVoxelCount; 
+    Index64                   mInterruptCount;
+    const double              mSpread;
+    math::Rand01<double, RandomGenerator> mRand01;
+
     /// This is a base class so the constructor is protected
     BasePointScatter(PointAccessorType& points,
                      RandomGenerator& randGen,
+                     double spread,
                      InterruptType* interrupt = NULL)
         : mPoints(points)
         , mInterrupter(interrupt)
         , mPointCount(0)
         , mVoxelCount(0)
         , mInterruptCount(0)
+        , mSpread(math::Clamp01(spread))
         , mRand01(randGen)
     {
     }
 
-    PointAccessorType&        mPoints;
-    InterruptType*            mInterrupter;
-    Index64                   mPointCount;
-    Index64                   mVoxelCount;
-    Index64                   mInterruptCount;
-    math::Rand01<double, RandomGenerator> mRand01;
-
     inline void start(const char* name)
     {
         if (mInterrupter) mInterrupter->start(name);
@@ -403,8 +412,12 @@ protected:
         return !(mInterruptCount++ & ((1<<5)-1)) && util::wasInterrupted(mInterrupter);
     }
 
-    inline double getRand() { return mRand01(); }
+    /// @brief Return a random floating point number between zero and one
+    inline double getRand01() { return mRand01(); }
 
+    /// @brief Return a random floating point number between 0.5 -+ mSpread/2
+    inline double getRand() { return 0.5 + mSpread * (mRand01() - 0.5); }
+   
     template <typename GridT>
     inline void addPoint(const GridT &grid, const Vec3R &dmin)
     {
@@ -414,7 +427,7 @@ protected:
         mPoints.add(grid.indexToWorld(pos));
         ++mPointCount;
     }
-
+   
     template <typename GridT>
     inline void addPoint(const GridT &grid, const Vec3R &dmin, const Coord &size)
     {
@@ -432,6 +445,6 @@ protected:
 
 #endif // OPENVDB_TOOLS_POINT_SCATTER_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
